from __future__ import absolute_import, print_function, unicode_literals

import apache_beam as beam

from dojo.dataset import Dataset

from .storage import BeamLocalFileStore, BeamGoogleStorageStore, BeamS3Store
from .transform import Conform, Validate, ConvertFrom, ConvertTo
from .runners.direct import DirectBeamRunner
from .runners.dataflow import DataflowBeamRunner


class BeamDataset(Dataset):

    RUNNERS = {
        'direct': DirectBeamRunner,
        'cloud': DataflowBeamRunner
    }

    STORES = {
        'file': BeamLocalFileStore,
        'gs': BeamGoogleStorageStore,
        's3': BeamS3Store
    }

    def run(self, pipeline):
        self.pipeline = pipeline
        inputs = self.inputs()
        rows = self.process(inputs)
        self.output(rows)
        return rows

    def read_input(self, rows, dataset_name):
        schema = self.input_schema(dataset_name)
        return (rows | 'from_json ' + dataset_name >> beam.ParDo(ConvertFrom())
                     | 'validate ' + dataset_name >> beam.ParDo(Validate(schema)))

    def write_output(self, rows, format):
        # The storage class is expected to do the ConvertTo transform.
        return (rows | 'conform ' >> beam.ParDo(Conform(self.OUTPUT))
                     | 'validate ' >> beam.ParDo(Validate(self.OUTPUT)))



#!/usr/bin/env python

from setuptools import setup, find_packages


setup(
    name='dojo-beam',
    version='0.0.1',
    description='Apache Beam adapters and datasets for the Dojo data framework.',
    author='Steven Normore',
    author_email='steven@dataup.io',
    url='https://dojo.dataup.io/',
    packages=find_packages(),
    scripts=[],
    install_requires=[
        'apache-beam'
    ]
)



from __future__ import absolute_import, print_function, unicode_literals

import os
import json
import random
import tempfile
import apache_beam as beam

from apache_beam.utils.pipeline_options import PipelineOptions, SetupOptions, StandardOptions
from dojo.runners.job import JobRunner


class GoogleCloudAuth(object):

    def __init__(self, secrets):
        self.secrets = secrets

    def __enter__(self):
        self.f = tempfile.NamedTemporaryFile('w')
        json.dump(self.secrets, self.f)
        self.f.flush()
        self.previous_credentrials_file = os.environ.get('GOOGLE_APPLICATION_CREDENTIALS')
        os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = self.f.name

    def __exit__(self, *args):
        if self.previous_credentrials_file:
            os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = self.previous_credentrials_file
        else:
            del os.environ['GOOGLE_APPLICATION_CREDENTIALS']
        self.f.close()


class DirectBeamRunner(JobRunner):

    def run(self, job, config):
        project_id = config['cloud']['project']

        dataset_name = job.config['name'].replace('_', '-')
        '%s--%s-%s' % (project_id, dataset_name, random.randint(1, 1000))

        with GoogleCloudAuth(job.secrets['store']['connection']):
            options = PipelineOptions()
            setup_options = options.view_as(SetupOptions)
            setup_options.save_main_session = True
            setup_options.setup_file = os.path.join(os.getcwd(), 'setup.py')

            options.view_as(StandardOptions).runner = str('DirectRunner')

            pipeline = beam.Pipeline(options=options)

            job.setup()
            job.run(pipeline)
            job.teardown()

            result = pipeline.run()
            result.wait_until_finish()
